'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (args, config) {
	var params = void 0;
	try {
		params = getParams(args, config);

		getReleaseActions().forEach(function (action) {
			action(params);
		});

		_logger2.default.success('WP.org release is created: ' + getSvnUrl(params.slug, params.version));
	} catch (e) {
		_logger2.default.error(e.toString());
	}

	afterRelease(params);
};

var _diff = require('diff');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _globAll = require('glob-all');

var _globAll2 = _interopRequireDefault(_globAll);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _path = require('path');

var _fileHeader = require('../utils/file-header');

var _fileHeader2 = _interopRequireDefault(_fileHeader);

var _git = require('../utils/git');

var _git2 = _interopRequireDefault(_git);

var _logger = require('../utils/logger');

var _logger2 = _interopRequireDefault(_logger);

var _readme = require('../utils/readme');

var _readme2 = _interopRequireDefault(_readme);

var _string = require('../utils/string');

var _string2 = _interopRequireDefault(_string);

var _svn = require('../utils/svn');

var _svn2 = _interopRequireDefault(_svn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getSvnUrl(slug, tag) {
	var url = 'https://plugins.svn.wordpress.org/' + slug;
	return tag ? url + '/' + tag : url;
}

/**
 * Internal dependencies
 */
/**
 * External dependencies
 */


function getParams(args, config) {
	var files = config.files || {};
	var mainFile = args.file || files.main;

	var buildFiles = getBuildFiles(files);
	var assets = getAssetFiles(files);

	var type = args.type || config.type || 'plugin';
	var wporg = config.wporg || {};
	var slug = args.slug || wporg.slug;
	if (!slug) {
		throw new Error('Missing wp.org slug. Specify it via --slug or define `wporg.slug` in .wpt.yml.');
	}

	var originalBranch = _git2.default.getCurrentBranch();
	var gh = config.gh || {};
	var branch = args.branch || gh.branch || 'master';

	var username = args.username || wporg.username;
	var buildPath = args.buildPath || wporg.buildPath;
	if (!buildPath) {
		throw new Error('Missing build path. Specify it via `wporg.buildPath` in .wpt.yml');
	}

	var data = _fileHeader2.default.getFileData(mainFile, type);
	var version = data.Version;

	return {
		originalBranch: originalBranch,
		branch: branch,
		slug: slug,
		username: username,
		version: version,
		buildPath: buildPath,
		buildFiles: buildFiles,
		assets: assets
	};
}

function getBuildFiles(configFiles) {
	var assets = getAssetFiles(configFiles);
	var src = configFiles.src;
	if (!src) {
		throw new Error('Missing files to build. Specify it via `files.src` in .wpt.yml.');
	}
	var filterNotAssets = function filterNotAssets(filepath) {
		var isAsset = false;
		for (var i = 0; i < assets.length; i++) {
			isAsset = _fsExtra2.default.statSync(filepath).isFile() && assets[i] === filepath;
			if (isAsset) {
				break;
			}
		}

		return !isAsset;
	};
	return _globAll2.default.sync(src).filter(filterNotAssets);
}

function getAssetFiles(configFiles) {
	var assets = _globAll2.default.sync(configFiles.assets || []);
	if (!assets) {
		throw new Error('Missing assets for wp.org. Specify it via `files.assets` in .wpt.yml.');
	}
	return assets;
}

function getReleaseActions() {
	return [switchGitBranch, checkBeforeCopy, cleanBuild, svnCheckout, copyFiles, svnCommitTrunk, svnCommitAssets, svnCreateTag, cleanBuild];
}

function switchGitBranch(params) {
	_git2.default.checkout(params.branch);
}

function checkBeforeCopy(params) {
	_git2.default.checkUncommittedChanges();
	_readme2.default.checkVersion(params.version);
}

function cleanBuild(params) {
	_rimraf2.default.sync(params.buildPath);
}

function svnCheckout(params) {
	_svn2.default.checkout(getSvnUrl(params.slug), rootBuild(params));
	_svn2.default.update(trunkDir(params), '--set-depth', 'infinity');
	_svn2.default.update(assetsDir(params), '--set-depth', 'infinity');
}

function rootBuild(params) {
	return (0, _path.resolve)(params.buildPath);
}

function trunkDir(params) {
	return (0, _path.resolve)(params.buildPath, 'trunk');
}

function assetsDir(params) {
	return (0, _path.resolve)(params.buildPath, 'assets');
}

function unbaseFiles(base, files) {
	var baseLen = _string2.default.untrailingslashit(base).length;
	return files.map(function (file) {
		var unbase = _string2.default.untrailingslashit(file).substring(baseLen);
		return '/' === unbase.charAt(0) ? unbase.substring(1) : unbase;
	});
}

function copyFiles(params) {
	// Remove all files in trunk and assets that are no longer exist.
	removeUnindexedInTrunk(params);
	removeUnindexedInAssets(params);

	var trunk = trunkDir(params);
	var assets = assetsDir(params);
	var copyToTrunk = copyFileCallback(trunk);
	var copyToAssets = copyFileCallback(assets, { copyDir: false });

	// Copy src files to trunk.
	params.buildFiles.forEach(copyToTrunk);

	(0, _mkdirp2.default)(assets);

	// Copy assets to svn assets.
	params.assets.forEach(copyToAssets);
}

function removeUnindexedInTrunk(params) {
	var svnTrunk = _globAll2.default.sync([_string2.default.trailingslashit(params.buildPath) + 'trunk/**']);
	var base = _string2.default.trailingslashit(params.buildPath) + 'trunk';
	var from = unbaseFiles(base, svnTrunk).filter(_string2.default.notEmpty);
	var to = params.buildFiles;

	removeFilesFromDiff((0, _path.resolve)(params.buildPath, 'trunk'), from, to);
}

function removeUnindexedInAssets(params) {
	var svnAssets = _globAll2.default.sync([_string2.default.trailingslashit(params.buildPath) + 'assets/**']);
	var base = _string2.default.trailingslashit(params.buildPath) + 'assets';
	var from = unbaseFiles(base, svnAssets).filter(_string2.default.notEmpty);
	var to = params.assets.map(function (f) {
		return (0, _path.basename)(f);
	});

	removeFilesFromDiff((0, _path.resolve)(params.buildPath, 'assets'), from, to);
}

function removeFilesFromDiff(base, from, to) {
	(0, _diff.diffArrays)(from, to).forEach(function (d) {
		if (d.removed && d.value) {
			d.value.forEach(function (f) {
				_rimraf2.default.sync((0, _path.resolve)(base, f));
			});
		}
	});
}

function copyFileCallback(dst, opts) {
	opts = opts || { copyDir: true };

	return function (filepath) {
		var stat = _fsExtra2.default.statSync(filepath);
		if (stat.isDirectory() && opts.copyDir) {
			(0, _mkdirp2.default)((0, _path.resolve)(dst, filepath));
		} else if (stat.isFile()) {
			_fsExtra2.default.copySync(filepath, opts.copyDir ? (0, _path.resolve)(dst, filepath) : (0, _path.resolve)(dst, (0, _path.basename)(filepath)));
		}
	};
}

function svnCommitTrunk(params) {
	var trunk = trunkDir(params);

	_svn2.default.missingItems(trunk).forEach(_svn2.default.del);
	_svn2.default.untrackedItems(trunk).forEach(_svn2.default.add);
	_svn2.default.commit(trunk, params.username, 'Updates trunk for ' + params.version);
}

function svnCommitAssets(params) {
	var assets = assetsDir(params);

	_svn2.default.missingItems(assets).forEach(_svn2.default.del);
	_svn2.default.untrackedItems(assets).forEach(_svn2.default.add);
	_svn2.default.update(assets, '--accept', 'mine-full');
	_svn2.default.commit(assets, params.username, 'Updates assets for ' + params.version);
}

function svnCreateTag(params) {
	var from = (0, _path.resolve)(params.buildPath, 'trunk');
	var to = (0, _path.resolve)(params.buildPath, 'tags', params.version);

	_svn2.default.update(to);
	_svn2.default.copy(from, to);

	// Copying the same src to dst in the second time will makes the whole trunk
	// directory appears in tag. Third time will cause E150002 -- directory
	// already exists.
	_svn2.default.del((0, _path.resolve)(to, 'trunk'));

	_svn2.default.commit(to, params.username, 'Creates tag ' + params.version);
}

function afterRelease(params) {
	try {
		_git2.default.checkout(params.originalBranch);
	} catch (e) {}
}
module.exports = exports['default'];