'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _promzard = require('promzard');

var _promzard2 = _interopRequireDefault(_promzard);

var _read = require('read');

var _read2 = _interopRequireDefault(_read);

var _logger = require('../utils/logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * External dependencies
 */
var defaultSrc = ['**', '!node_modules/**', '!vendor/**', '!tests/**', '!test/**', '!build/**', '!tmp/**', '!README.md'];

/**
 * Internal dependencies
 */


var defaultAssets = ['assets/*.png', 'assets/*.svg'];

function read(configPath) {
	var configFile = configPath || _path2.default.join(process.cwd(), '.wpt.yml');
	var configExists = _fs2.default.existsSync(configFile);
	var config = {};

	try {
		if (configExists) {
			config = _jsYaml2.default.safeLoad(_fs2.default.readFileSync(configFile, 'utf8')) || {};
		}
	} catch (e) {
		throw new Error('Something wrong when reading ' + configFile + '.\n' + e.toString());
	}

	return config;
}

function generate(defaults, useDefaults) {
	var input = _path2.default.resolve(__dirname, '../init-input.js');
	var ctx = {
		defaults: defaults,
		useDefaults: useDefaults
	};

	(0, _promzard2.default)(input, ctx, generateCb);
}

function generateCb(err, data) {
	if (err) {
		return _logger2.default.warning(err.message);
	}

	try {
		var dst = _path2.default.resolve(process.cwd(), '.wpt.yml');
		var yml = prepareYaml(data);

		promptWrite(dst, yml);
	} catch (e) {
		_logger2.default.error(e.message);
	}
}

function prepareYaml(data) {
	var yml = {
		type: data.type
	};

	var hasValue = function hasValue(v) {
		return undefined !== v && null !== v;
	};
	var isEmpty = function isEmpty(o) {
		return JSON.stringify(o) === JSON.stringify({});
	};

	var files = {};
	if (hasValue(data.src)) {
		files.src = data.src;
	}
	if (hasValue(data.main)) {
		files.main = data.main;
	}
	if (hasValue(data.assets)) {
		files.assets = data.assets;
	}
	if (!isEmpty(files)) {
		yml.files = files;
	}

	var gh = {};
	if (hasValue(data.ghRepo)) {
		gh.repo = data.ghRepo;
	}
	if (hasValue(data.ghUser)) {
		gh.username = data.ghUser;
	}
	if (!isEmpty(gh)) {
		yml.gh = gh;
	}

	var wporg = {};
	if (hasValue(data.wporgSlug)) {
		wporg.slug = data.wporgSlug;
	}
	if (hasValue(data.wporgUser)) {
		wporg.username = data.wporgUser;
	}
	if (hasValue(data.wporgBuildPath)) {
		wporg.buildPath = data.wporgBuildPath;
	}
	if (!isEmpty(wporg)) {
		yml.wporg = wporg;
	}

	return _jsYaml2.default.safeDump(yml, { indent: 4 });
}

function promptWrite(dst, yml) {
	_logger2.default.info('About to write %s: \n\n%s\n', dst, yml);
	(0, _read2.default)({ prompt: 'Is this ok?', 'default': 'yes' }, function (err, ok) {
		if (err) {
			return _logger2.default.error(err.message);
		}
		var cancel = !ok || 'y' !== ok.toLowerCase().charAt(0);

		return cancel ? _logger2.default.warning('Aborted.') : write(dst, yml);
	});
}

function write(configPath, yml) {
	_fs2.default.writeFile(configPath, yml, 'utf8', function (err) {
		if (err) {
			_logger2.default.error(err.message);
		}
	});
}

exports.default = {
	read: read,
	generate: generate,
	defaultSrc: defaultSrc,
	defaultAssets: defaultAssets
};
module.exports = exports['default'];